function varargout = yearly_risk(fid, varargin)
fid = strcat(fid,'_yearly_risk');
% If a function name is given, use it to call subfunction.
fh = str2func(fid);
[varargout{1:nargout}] = fh(varargin{:});

function []=yearly_risk(varargin)  % This was the original myFunc
ME_yearly_risk(varargin);
SVM_yearly_risk(varargin);
RF_yearly_risk(varargin);
RT_yearly_risk(varargin);


yearly_risk('ME_yearly_risk');
yearly_risk('SVM_yearly_risk');
yearly_risk('RF_yearly_risk');
yearly_risk('RT_yearly_risk');

end
end