function varargout = PRESULT(fid, varargin)
% If a function name is given, use it to call subfunction.
fh = str2func(fid);
[varargout{1:nargout}] = fh(varargin{:});

function []=presult(varargin)  % This was the original myFunc
train(varargin);
test(varargin);
train_test1(varargin);
train_test2(varargin);
kfold(varargin);
train_abs_risk(varargin);
yearly_risk(varargin)

presult('train');
presult('test');
presult('train_test1');
presult('train_test2');
presult('kfold');
presult('train_abs_risk');
presult('yearly_risk');

end
end