function varargout = test(fid, varargin) % If a function name is given, use it to call subfunction.
fid = strcat(fid,'_test');
fh = str2func(fid);
[varargout{1:nargout}] = fh(varargin{:});

function []=train(varargin)  % This was the original myFunc
ME_test(varargin);
SVM_test(varargin);
RF_test(varargin);
RT_test(varargin);

test('ME_test');
test('SVM_test');
test('RF_test');
test('RT_test');
end
end