function varargout = train_test2(fid, varargin)
fid = strcat(fid,'_train_test2');
% If a function name is given, use it to call subfunction.
fh = str2func(fid);
[varargout{1:nargout}] = fh(varargin{:});

function []=train_test2(varargin)  % This was the original myFunc
ME_train_test2(varargin);
SVM_train_test2(varargin);
RF_train_test2(varargin);
RT_train_test2(varargin);

train_test2('ME_train_test2');
train_test2('SVM_train_test2');
train_test2('RF_train_test2');
train_test2('RT_train_test2');
end
end