function sss = raw_norm(varargin)
p = inputParser;
addParamValue(p,'raw_data','PID_new.csv',@ischar);
new_array = varargin(:);
%disp (varargin(:));
for i =1:size(new_array,1)
     for j = 1:size(p.Parameters,2)
         param=p.Parameters{j};
         array_entry=new_array{i};
         first_check=strcat('-',param);
         second_check=strcat('--',param);
         if strcmp(array_entry,first_check)
            new_array{i}=param;
         end
         if strcmp(array_entry,second_check)
            new_array{i}=param;
         end
     end
end

new_array(strcmp('yes',new_array))={'shuffle'};
new_array(strcmp('no',new_array))={'default'};

parse(p,new_array{:});
%disp (p.Parameters);
%disp (p.UsingDefaults);

data_file = (p.Results.raw_data);
sss = perl('normalization.pl', '--input', data_file, '--output', 'few_data_norm', '--csv');
% data_file = load(data_file);
% disp (data_file);
% raws_to_norm = data_file(:,1:end-1);
% decision_column = data_file(:,end);
% save('train_data_a.txt', 'x', '-ascii');
% for i = 1:size(raws_to_norm,1)
%     for j = 1:
end