function varargout = train_abs_risk(fid, varargin)
fid = strcat(fid,'_train_abs_risk');
% If a function name is given, use it to call subfunction.
fh = str2func(fid);
[varargout{1:nargout}] = fh(varargin{:});

function []=train_abs_risk(varargin)  % This was the original myFunc
ME_train_abs_risk(varargin);
SVM_train_abs_risk(varargin);
RF_train_abs_risk(varargin);
RT_train_abs_risk(varargin);


train_abs_risk('ME_train_abs_risk');
train_abs_risk('SVM_train_abs_risk');
train_abs_risk('RF_train_abs_risk');
train_abs_risk('RT_train_abs_risk');
end
end